/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __GENOMEBLUEPRINT__
#define __GENOMEBLUEPRINT__

#ifndef __GENE__
#include "gene.h"
#endif

typedef double (*GenomeFitnessProc)( unsigned inNumGenes, const Gene *inGene, void *inFitnessData);

// This keeps up with number of, type, min, and max values
// for genomes in a central place, as well
// as what mutation and crossover operations are supported.
// also keeps up with fitness evaluation function.
class GenomeBlueprint {
public:
	GenomeBlueprint(void);
	~GenomeBlueprint();

	//// Functions to setup values ////
	
	// set number of genes, call this first because it zeroes them all
	// returns false if fails.
	bool SetNumGenes(unsigned inNumGenes);

	// set min and max (and implicitly the type) of a gene
	void SetGeneMinMax(unsigned index, const Gene& inMin, const Gene& inMax);

	// set supported mutation operations
	void SetUniformMutate(bool inUniformMutate) { mUniformMutate = inUniformMutate; }
	void SetGaussianMutate(bool inGaussianMutate) { mGaussianMutate = inGaussianMutate; }
	void SetBoundaryMutate(bool inBoundaryMutate) { mBoundaryMutate = inBoundaryMutate; }

	// set supported crossover operations
	void SetSimpleCrossover(bool inSimpleCrossover) { mSimpleCrossover = inSimpleCrossover; }
	void SetArithmeticCrossover(bool inArithmeticCrossover) { mArithmeticCrossover = inArithmeticCrossover; }
	void SetHeuristicCrossover(bool inHeuristicCrossover) { mHeuristicCrossover = inHeuristicCrossover; }

	// set mutation probability
	void SetMutationProb(double inMutationProb) { mMutationProb = inMutationProb; }
	// set crossover probability
	void SetCrossoverProb(double inCrossoverProb) { mCrossoverProb = inCrossoverProb; }

	// set fitness proc and extra data pointer
	void SetFitnessProc(GenomeFitnessProc inFitnessProc, void *inFitnessData) 
		{ mFitnessProc = inFitnessProc; mFitnessData = inFitnessData; }

	// Accessors
	unsigned GetNumGenes(void) const { return mNumGenes; }

	bool IsReal(unsigned index) const { return mMin[index].IsReal(); }
	bool IsInteger(unsigned index) const { return mMin[index].IsInteger(); }

	const Gene& GetMin(unsigned index) const { return mMin[index]; }
	const Gene& GetMax(unsigned index) const { return mMax[index]; }

	bool GetUniformMutate(void) const { return mUniformMutate; }
	bool GetGaussianMutate(void) const { return mGaussianMutate; }
	bool GetBoundaryMutate(void) const { return mBoundaryMutate; }

	bool GetSimpleCrossover(void) const { return mSimpleCrossover; }
	bool GetArithmeticCrossover(void) const { return mArithmeticCrossover; }
	bool GetHeuristicCrossover(void) const { return mHeuristicCrossover; }

	double GetGaussianFactor(void) const { return mGaussianFactor; }
	void SetGaussianFactor(double inGaussianFactor) { mGaussianFactor = inGaussianFactor; }

	double GetMutationProb(void) const { return mMutationProb; }
	double GetCrossoverProb(void) const { return mCrossoverProb; }

	// returns # of different mutation operations
	unsigned GetNumMutationTypes(void) const;

	// returns # of different crossover operations
	unsigned GetNumCrossoverTypes(void) const;

	// get fitness proc
	GenomeFitnessProc GetFitnessProc(void) const { return mFitnessProc; }
	// get extra data ptr passed to fitness proc
	void *GetFitnessData(void) const { return mFitnessData; }

protected:
	unsigned mNumGenes; // number of genes per genome
	// min is used for types of parameters, min/max are used for range.
	Gene *mMin, *mMax;
	// flags for what mutation and crossover operations are supported.
	bool mUniformMutate, mGaussianMutate, mBoundaryMutate;
	bool mSimpleCrossover, mArithmeticCrossover, mHeuristicCrossover;
	double mGaussianFactor; // used by gaussianmutate, default of 1

	double mMutationProb, mCrossoverProb;
	GenomeFitnessProc mFitnessProc;
	void *mFitnessData;
};

#endif
