/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __MULTIPOPULATION__
#include "multipopulation.h"
#endif

#ifndef __MATHUTILS__
#include "mathutils.h"
#endif

const unsigned kDefaultCycles = 100;

/*
	unsigned mNumPop;
	Population **mPop; // populations
	unsigned mCycles;
*/

MultiPopulation::MultiPopulation( void )
{
	mCycles = kDefaultCycles;
}

MultiPopulation::~MultiPopulation()
{
	Reset();
}

// add populations, must have same blueprint for genomes, but
// population parameters can be different.
void MultiPopulation::AddPopulation(Population *inPop)
{
	try
	{
		mPop.push_back(inPop);
	}
	catch(...)
	{
		delete inPop; // free first
		throw;
	}
}


// crosspolinate between populations, then evolve each population mCycles times
void MultiPopulation::Evolve(void)
{
	CrossPollinate();
	unsigned cycles = mCycles;
	while(cycles--)
	{
		for(unsigned i = 0;i<GetNumPop();i++)
		{
			Population *pop = GetPop(i);
			pop->Evolve();
		}
	}
}

// Get winner from among all populations.
const Genome *MultiPopulation::GetWinner(void) const
{
	const Genome *winner = 0;
	for(unsigned i = 0;i<GetNumPop();i++)
	{
		const Population *pop = GetPop(i);
		unsigned index = pop->GetWinner();
		const Genome *genome = pop->GetGenome(index);
		if( !winner || genome->GetFitness() > winner->GetFitness() )
			winner = genome;
	}
	return winner;
}

void MultiPopulation::Reset(void)
{
	for(unsigned i = 0;i<GetNumPop();i++)
		delete GetPop(i);

	mPop.clear();
}

void MultiPopulation::CrossPollinate(void)
{
	// randomly swap genetic matter between populations.

	// don't know the best way or amount of this to do, so just trying something relatively
	// convenient to implement.

	Population *pop, *pop1, *pop2;
	long smSize = -1, sz;
	long i,j, index1, index2;
	unsigned numPop = GetNumPop();
	Genome *g;

	// find size of smallest population as a reference for how much material to swap around
	for(i = 0;i<numPop;i++)
	{
		pop = GetPop(i);
		sz = pop->GetSize();
		if( smSize == -1 || sz < smSize )
			smSize = sz;
	}

	for(j=0;j<smSize;j++)
	{
		pop1 = GetPop( RandLongMax(numPop) );
		pop2 = GetPop( RandLongMax(numPop) );
		if( pop1 != pop2 )
		{
			// swap a genome between these two.
			index1 = RandLongMax(pop1->GetSize());
			index2 = RandLongMax(pop2->GetSize());
			g = pop1->GetGenome(index1); // get 1st
			g = pop2->SwapGenome(index2,g); // swap in 2nd
			(void)pop1->SwapGenome(index1,g); // replace in 1st

		}
	}

}
