/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __GENOME__
#include "genome.h"
#endif

#ifndef __MATHUTILS__
#include "mathutils.h"
#endif

Genome::Genome( GenomeBlueprint *inBlueprint )
: mFitness(0.0), mGene(0), mBlueprint(inBlueprint)
{
	// allocate mGene
	mGene = new Gene[ inBlueprint->GetNumGenes() ];
	// should throw if fails
}


Genome::~Genome(void)
{
	if( mGene )
	{
		delete[] mGene; 
		mGene = 0;
	}
}



// this you don't override
void Genome::CalcFitness(void)
{
	mFitness = Fitness();
}

///// overide these to implement a Genome
// return a bare (unspecified) specimen, this is use to create initial population,
// and new random members of population
Genome *Genome::Spawn(void)
{
	Genome *ptr = new Genome(mBlueprint);
	return ptr;
}


// make this one random
void Genome::Random(void)
{
	for(unsigned i=0; i<mBlueprint->GetNumGenes(); i++)
		mGene[i] = UniformMutate( mBlueprint->GetMin(i), mBlueprint->GetMax(i) );
}

// make this one all zero
void Genome::Zero(void)
{
	for(unsigned i=0; i<mBlueprint->GetNumGenes(); i++)
	{
		Gene zero = mBlueprint->GetMin(i);
		zero.Zero();
		mGene[i] = zero;
	}
}


void Genome::Copy(const Genome& inGenome)
{
	mBlueprint = inGenome.mBlueprint;

	for(unsigned i=0;i<mBlueprint->GetNumGenes();i++)
		mGene[i] = inGenome.mGene[i];	

	mFitness = inGenome.mFitness;
}


Genome *Genome::Copy(void)
{
	Genome *ptr = new Genome(mBlueprint);
	if( !ptr )
		return 0;

	ptr->Copy(*this);

	return ptr;	
}

Gene Genome::AverageGenes(Gene a, Gene b)
{
	Gene ret;
	if( a.IsReal() )
		ret.SetReal( (a.GetReal() + b.GetReal()) / 2.0 );
	else
		ret.SetInteger( (a.GetInteger() + b.GetInteger()) / 2 );
	return ret;
}

Gene Genome::ProjectGene(
	Gene better, Gene worse, 
	Gene min, Gene max,
	double vlen )
{
	Gene ret;

	if( better.IsInteger() )
	{
		long worsel, betterl;
		worsel = worse.GetInteger();
		betterl = better.GetInteger();

		ret.SetInteger( (long)CLIP<double>( 
			(double)worsel + vlen * (double)(betterl - worsel) + 0.5, 
			(double)min.GetInteger(), (double)max.GetInteger() ) 
			);
	}
	else
	{
		double worsed, betterd;
		worsed = worse.GetReal();
		betterd = better.GetReal();
		ret.SetReal( 
			CLIP<double>( worsed + vlen * (betterd - worsed), 
			min.GetReal(), max.GetReal() )
			);
	}

	return ret;
}

void Genome::SimpleCrossover(Genome *child, Genome *mother, Genome *father)
{
	// do a 2-point crossover that selectes some values
	// from the parent and some values from the child
	
	// 2 random numbers < paramCount
	long start,stop,i;

	// start by setting it to a copy of mother.
	child->Copy(*mother);

	// then replace a random contiguous portion
	// with the values from father
	
	start = RandLongMax(mBlueprint->GetNumGenes());
	stop = RandLongMax(mBlueprint->GetNumGenes());
	if( stop < start )
	{
		i = stop;
		stop = start;
		start = i;
	}

	i=start;
	while(i!=stop)
	{
		child->mGene[i] = father->mGene[i];

		i++;
		if( i == (long)mBlueprint->GetNumGenes() )
			i = 0; // doing mod arithmetic on counter
	}
	/*
	// Single point crossover code.
	// do a 1 point crossover
	// crossover with another and return the new one
	unsigned i, point = rand() % mBlueprint->GetNumGenes();

	for(i=0;i<point;i++)
		child->mGene[i] = mother->mGene[i];
	for(i=point;i<mBlueprint->GetNumGenes();i++)
		child->mGene[i] = father->mGene[i];
	*/
}

void Genome::ArithmeticCrossover(Genome *child, Genome *mother, Genome *father)
{
	// do an arithmetic crossover , averages all the values
	// from the parents to make the children
	for(unsigned i=0;i<mBlueprint->GetNumGenes();i++)
	{
		child->mGene[i] = AverageGenes(mother->mGene[i], father->mGene[i]);
	}
}


void Genome::HeuristicCrossover(Genome *child, Genome *mother, Genome *father)
{
	double vlen = Random1() + 1.0; // between 1 and 2
	
	// Need ptr to better and ptr to worst
	Genome *better, *worse;
	if( mother->GetFitness() > father->GetFitness() )
	{
		better = mother;
		worse = father;
	}
	else
	{
		better = father;
		worse = mother;
	}

	for(unsigned i=0;i<mBlueprint->GetNumGenes();i++)
	{
		child->mGene[i] = ProjectGene(better->mGene[i],worse->mGene[i],
			mBlueprint->GetMin(i),mBlueprint->GetMax(i),vlen);
	}
}



// crossover with another and return the new one
Genome *Genome::Crossover(Genome *inG)
{
	Genome *ptr = new Genome(mBlueprint);
	if( !ptr )
		return 0;
	
	bool canUse[3];
	canUse[0] = mBlueprint->GetSimpleCrossover();
	canUse[1] = mBlueprint->GetArithmeticCrossover();
	canUse[2] = mBlueprint->GetHeuristicCrossover();
	
	unsigned canCount = mBlueprint->GetNumCrossoverTypes();
	if( !canCount )
	{
		// set to copy of selves if crossover not supported
		ptr->Copy(*this);
	}
	else
	{
		long i, index = RandLongMax(canCount);

		for(i=0;i<3;i++)
		{
			if( canUse[i] && index == 0 )
				break;
			
			index -= canUse[i];
		}

		switch(i)
		{
		case 0:
			SimpleCrossover(ptr,this,inG);
			break;
		case 1:
			ArithmeticCrossover(ptr,this,inG);
			break;
		case 2:
			HeuristicCrossover(ptr,this,inG);
			break;
		}
	}
	return ptr;
}



// Possible kinds of mutation
// flip bit - only good for binary genes
// uniform - random between min and max
// nonuniform - likelihood that value of mutation is 0 increases each generation
// gaussian - add a guassian distributed random number to the value
// bounding, randomly set to min or max.



Gene Genome::GaussianMutate( Gene value, Gene min, Gene max )
{
	Gene ret;
	double factor = mBlueprint->GetGaussianFactor();;
	if( factor == 0.0 )
	{	
		// special case.
		factor = value.GetReal();
		if( factor == 0.0 )
			factor = 1.0;
	}

	double r = GaussianRandom() * factor;

	if( min.IsInteger() )
	{
		ret.SetInteger( 
			(long)CLIP<double>( 
				(double)value.GetInteger() + r + 0.5, 
				(double)min.GetInteger(), (double)max.GetInteger() )
			);
	}
	else
	{
		ret.SetReal(
			CLIP<double>(value.GetReal() + r, min.GetReal(), max.GetReal() )
			);
	}

	return ret;
}


// get randome value for a parameter
Gene Genome::UniformMutate(Gene min, Gene max)
{
	Gene ret;

	if( min.IsInteger() )
	{
		ret.SetInteger( (long)( Random1() * (double)(max.GetInteger() - min.GetInteger()) + (double)min.GetInteger() +0.5) );
	}
	else
	{
		ret.SetReal( Random1() * (max.GetReal() - min.GetReal()) + min.GetReal() );
	}

	return ret;
}


Gene Genome::BoundaryMutate(Gene min, Gene max)	
{
	return RandomBool() ? min : max;
}

// mutate this one, meaning (possibly) change it slightly.
void Genome::Mutate(void)
{	
	for(unsigned i=0; i<mBlueprint->GetNumGenes(); i++)
	{
		if( Random1() < mBlueprint->GetMutationProb() )
		{
			bool canUse[3];
			canUse[0] = mBlueprint->GetUniformMutate();
			canUse[1] = mBlueprint->GetGaussianMutate();
			canUse[2] = mBlueprint->GetBoundaryMutate();
			
			unsigned canCount = mBlueprint->GetNumCrossoverTypes();
			if( canCount )
			{
				// choose type of mutation.
				long index = RandLongMax(canCount);

				for(unsigned j=0;j<3;j++)
				{
					if( canUse[j] && index == 0 )
						break;
					
					index -= canUse[j];
				}


				switch(j)
				{
				case 0:
					mGene[i] = UniformMutate(mBlueprint->GetMin(i),mBlueprint->GetMax(i));
					break;
				case 1:
					mGene[i] = GaussianMutate(mGene[i],mBlueprint->GetMin(i),mBlueprint->GetMax(i));
					break;
				case 2:
					mGene[i] = BoundaryMutate(mBlueprint->GetMin(i),mBlueprint->GetMax(i));
					break;
				}

			}
		}
	}
}


// return fitness of this specimen
double Genome::Fitness(void)
{
	double fitness = 0.0;

	if( mBlueprint->GetFitnessProc() )
		fitness = mBlueprint->GetFitnessProc()( mBlueprint->GetNumGenes(), mGene, mBlueprint->GetFitnessData() );

	return fitness;
}
